/// <reference path="../src/global.d.ts" />
export interface CustomObjectFormatter<T = any> {
    /**
     * Returns a single-line summary of the object or null.
     * If null is returned, the default format will be used.
     * @param object
     * @param config
     * You can specify “config” attribute in object tag and assign it
     * to an arbitrary object, this object will be passed here as second parameter.
     */
    header(object: unknown, config?: T): JSX.Element | null;
    /**
     * Returns true if the object can be opened to show more detail.
     * @param object
     * @param config same as in header function.
     */
    hasBody(object: unknown, config?: T): boolean;
    /**
     * Returns the expanded version of the object, to be displayed after it's opened.
     * @param object
     * @param config same as in header function.
     */
    body(object: unknown, config?: T): JSX.Element;
}
/**
 * Install a Custom Object Formatter
 * @param formatter The formatter
 */
export declare function installCustomObjectFormatter(formatter: CustomObjectFormatter): void;
export declare type Variants = keyof typeof darkTheme;
export declare type CSSProperties = import('csstype').PropertiesHyphen | import('csstype').Properties;
export interface ElementAttributes {
    /**
     * CSS of this element.
     *
     * Google's original proposal does not contains
     * the object type `CSS.Properties`.
     *
     * It is handled and transformed by this library.
     */
    style?: string | CSSProperties;
    children?: JSX.Element[];
    /**
     * onClick event of the element.
     */
    onClick?(): void;
    variant?: Variants[];
}
export interface ImageElementAttributes extends ElementAttributes {
    src: string;
    width?: number | string;
    height?: number | string;
}
export interface ObjectElementAttributes {
    object?: unknown;
    config?: any;
}
export declare const Fragment = "span";
export declare function isJSXElement(x: unknown): x is JSX.Element;
export declare function createElementTyped(tag: keyof JSX.IntrinsicElements | ((props: any) => JSX.Element), _props: ElementAttributes | ObjectElementAttributes | null, ..._: JSX.Element[]): JSX.Element;
export declare const createElement: (tag: string, props: any, ...children: any[]) => JSX.Element;
export declare function useState<State, T extends object = object>(bindingObject: T, initialState: (obj: T) => State): readonly [Readonly<State>, (nextState: Partial<Readonly<State>>) => void, () => void];
declare const darkTheme: {
    propertyPreviewName: {
        color: string;
    };
    functionPrefix: {
        color: string;
    };
    propertyName: {
        color: string;
    };
    null: {
        color: string;
    };
    bigint: {
        color: string;
    };
    number: {
        color: string;
    };
    string: {
        color: string;
        whiteSpace: string;
        'unicode-bidi': string;
    };
    quote: {
        color: string;
    };
    node: {
        color: string;
    };
    fade: CSSProperties;
    code: import("csstype").Properties<0 | (string & {}), string & {}>;
};
export {};
//# sourceMappingURL=index.d.ts.map